from os import PathLike
from pydrive2.auth import GoogleAuth
from pydrive2.drive import GoogleDrive
from pydrive2.files import GoogleDriveFile

from py_saver.model.google_drive.permission import GoogleDrivePermission
from py_saver.sav_uploader.uploader import Uploader


class GoogleDriveUploader(Uploader):
    def __init__(self, path_to_json_service_account: PathLike[str],
                 share_with: GoogleDrivePermission | None = None):
        self._path_to_json_service_account = path_to_json_service_account
        self._share_with: GoogleDrivePermission = share_with

    def _login_with_service_account(self) -> GoogleAuth:
        settings = {
            "client_config_backend": "service",
            "service_config": {
                "client_json_file_path": f"{self._path_to_json_service_account}",
            }
        }
        gauth = GoogleAuth(settings=settings)
        gauth.ServiceAuth()
        return gauth

    def delete_all_files_of_this_service_account(self):
        gauth = self._login_with_service_account()
        google_drive = GoogleDrive(gauth)
        file_list = google_drive.ListFile({'q': "'root' in parents or trashed=false"}).GetList()
        if not file_list:
            return
        for file in file_list:
            try:
                print(f"Deleting file: {file['title']} (ID: {file['id']})")
                file.Delete()
            except Exception as e:
                print(f"Failed to delete file: {file['title']} - {str(e)}")

    def upload_sav_in_google(self):
        gauth = self._login_with_service_account()
        google_drive = GoogleDrive(gauth)
        with self.zip_save_dir() as file_to_save_in_google:
            file = google_drive.CreateFile({'title': f'{file_to_save_in_google.name}'})
            file.SetContentFile(file_to_save_in_google)
            file.Upload()
            if self._share_with:
                self._share_file(file)

    def _share_file(self, file_to_share: GoogleDriveFile) -> None:
        file_to_share.InsertPermission(self._share_with.model_dump(by_alias=True, exclude_none=True))



    def upload_sav(self) -> None:
        self.upload_sav_in_google()

