import os
from abc import ABC
from contextlib import contextmanager
from datetime import datetime
from typing import Iterator
from pathlib import Path
import shutil


class Uploader(ABC):
    dir_to_save_path = os.getenv('SAVER_DIR', 'sav')

    @contextmanager
    def zip_save_dir(self) -> Iterator[Path]:
        creation_date = f'{datetime.now()}'
        zip_filename = f'sav_of_{creation_date}'.replace(' ', '_')
        shutil.make_archive(zip_filename, 'zip',
                            root_dir=self._get_dir_saved_path(), base_dir=self._get_dir_saved_path())
        path_to_zip_file = Path.joinpath(Path.cwd(), f'{zip_filename}.zip')
        yield path_to_zip_file
        os.remove(path_to_zip_file)

    @classmethod
    def _get_dir_saved_path(cls) -> Path:
        return Path.joinpath(Path.cwd(), cls.dir_to_save_path)