import os
import shutil
from pathlib import Path
import subprocess

from py_saver.model.exception.not_have_dependancy import NotHaveEnougthDependancyException
from py_saver.saver.base_saver import BaseSaver


class ElasticSaver(BaseSaver):

    def __init__(self, unique_name: str, elastic_url: str, index_name: str, port: int, username: str, password: str,
                 default_command: str = 'elasticdump', rate_document: int = 100):
        self._unique_name = unique_name
        self._elastic_url = elastic_url
        self._index_name = index_name
        self._username = username
        self._password = password
        self._port = port
        self._default_command = default_command
        self._rate_limit = rate_document

    # TODO : check the file not exist !

    def save(self) -> Path:
        try:
            if not self._check_if_computer_have_dependency():
                raise NotHaveEnougthDependancyException('please install elasticdump program')
            path_to_save = Path.joinpath(self._get_dir_save_path(), f"{self._unique_name}")
            self._initialize_dir(self._get_dir_save_path())
            input_array_cmd = self._get_input_command_array()
            output_array_cmd = self._get_output_command_array(path_to_save)
            subprocess.run([f"{self._default_command}", *input_array_cmd, *output_array_cmd,
                            f"--limit={self._rate_limit}"])
            return path_to_save
        except subprocess.CalledProcessError as e:
            print(e.stderr)
            raise e

    # TODO: build url according to have username / pass and protocol scheme https/http
    def _get_input_command_array(self) -> list[str]:
        return [f"--input=https://{self._username}:{self._password}@{self._elastic_url}:{self._port}/{self._index_name}"]

    def _get_output_command_array(self, save_file_path: Path) -> list[str]:
        return [f"--output={save_file_path}"]

    def _check_if_computer_have_dependency(self) -> bool:
        try:
            subprocess.run([f"{self._default_command}", "--help"], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                           check=True)
            return True
        except (subprocess.CalledProcessError, FileNotFoundError) as e:
            return False
