import os
import shutil
from pathlib import Path
import subprocess

from py_saver.model.exception.not_have_dependancy import NotHaveEnougthDependancyException
from py_saver.saver.base_saver import BaseSaver


class MinioSaver(BaseSaver):

    def __init__(self, unique_name: str, minio_url: str, bucket_name: str, username: str, password: str,
                 default_command: str = 'mc'):
        self._unique_name = unique_name
        self._minio_url = minio_url
        self._bucket_name = bucket_name
        self._username = username
        self._password = password
        self._default_command = default_command

    def save(self) -> Path:
        try:
            if not self._check_if_computer_have_dependency():
                raise NotHaveEnougthDependancyException('please install mc program')
            _inialize_mc_output = subprocess.run(
                [f"{self._default_command}", "alias", "set", self._unique_name, self._minio_url, self._username,
                 self._password],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
            self._initialize_dir(self._get_dir_save_path())
            path_to_save = Path.joinpath(self._get_dir_save_path(), f"{self._unique_name}")
            _mc_save_output = subprocess.run(
                [f"{self._default_command}", "mirror", f"{self._unique_name}/{self._bucket_name}", f"{path_to_save}"],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
            return path_to_save
        except subprocess.CalledProcessError as e:
            print(e.stderr)
            raise e

    def _check_if_computer_have_dependency(self) -> bool:
        try:
            subprocess.run([f"{self._default_command}", "--version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                           check=True)
            return True
        except (subprocess.CalledProcessError, FileNotFoundError) as e:
            return False
