import os
from pathlib import Path
import subprocess

from py_saver.model.exception.not_have_dependancy import NotHaveEnougthDependancyException
from py_saver.saver.base_saver import BaseSaver


class MySqlSaver(BaseSaver):

    def __init__(self, unique_name: str, sql_host: str, database_name: str, username: str, password: str,
                 default_command='mysqldump'):
        self._unique_name = unique_name
        self._sql_host = sql_host
        self._database_name = database_name
        self._username = username
        self._password = password
        self._default_command = default_command

    def save(self) -> Path:
        try:
            if not self._check_if_computer_have_dependency():
                raise NotHaveEnougthDependancyException('please install mysqldump program')
            default_directory_to_save_file = self._get_dir_save_path()
            self._initialize_dir(default_directory_to_save_file)
            path_to_save_file = Path.joinpath(default_directory_to_save_file, f"{self._unique_name}")
            subprocess.run([f"{self._default_command}", "-h", f"{self._sql_host}",
                            "-u", f"{self._username}",
                            "-r", f"{str(path_to_save_file)}",
                            f"{self._database_name}"],
                           env={'MYSQL_PWD': self._password},
                           stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
            return path_to_save_file
        except subprocess.CalledProcessError as e:
            print(e.stderr)
            raise e

    def _check_if_computer_have_dependency(self) -> bool:
        try:
            subprocess.run([f"{self._default_command}", "--version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                           check=True)
            return True
        except (subprocess.CalledProcessError, FileNotFoundError) as e:
            return False
